// @ts-ignore
import yaml from 'js-yaml';
import {GenerateSettings, RawFileData} from "./src/types";
import {readFileSync,writeFileSync, existsSync, mkdirSync} from "fs";
import {parse} from "node:url";
if(!existsSync('generated')) mkdirSync('generated');
const yamlContent: RawFileData = yaml.load(readFileSync('data.yml', 'utf8'));
const generateSettings: GenerateSettings = yaml.load(readFileSync('codeGenSettings.yml', 'utf8'));
const icons= [
    ...yamlContent.items
          .map(item=>
            typeof item?.images?.thumbnail=='object'
              ?item.images!.thumbnail.icon
              :null),
    ...yamlContent.tags.map(tag=>tag.icon)
]
    .filter(item=>item) //remove null
    .filter((e, i, a) => a.indexOf(e) == i) //remove duplicates
    .join(',')
const code=`/* This file is auto-generated with codeGen.ts. DO NOT MANUALLY EDIT! */
import {${icons}} from "@mdi/js";
const icons = {${icons}} as const;
export default function (iconName: string) {
  const icon = iconName as keyof typeof icons;
  if(!icons[icon]) throw new Error('Icon not found: '+icon);
  return icons[icon];
}`
writeFileSync('generated/icons.ts', code, 'utf8')
let title=yamlContent.translations.title;
if(typeof title==="object") title=title.en;
const manifest={
    short_name: title,
    name: title,
    icons: [
        {
            src: "/logo.svg",
            type: "image/svg+xml",
            sizes: "512x512",
        },
        {
            src: "/logo.png",
            type: "image/png",
            sizes: "512x512",
            purpose: "any maskable",
        },
    ],
    id: "/",
    start_url: "/",
    background_color: "#3367D6",
    display: "standalone",
    scope: "/",
    theme_color: "#3367D6",
    description: title,
}
writeFileSync('public/manifest.json', JSON.stringify(manifest,null,2), 'utf8')
async function getCommitDateForProject(project: string, user: string) {
    const response = await fetch(`https://gitlab.rubend.nl/api/v4/projects/${project}/repository/commits?author=${user}&with_stats=true`)
    const data: {
        committed_date: string;
        title: string;
        stats: {
            total: number;
        }
    }[] = await response.json();
    const filters = generateSettings.gitCommitFilters.map(filter=>filter.toLowerCase());
    const filtered = data.filter(item=> {
        if(item.stats.total==0) return;
        const lowercase = item.title.toLowerCase();
        return !filters.some(filter=>lowercase.includes(filter))
    })
    if(filtered.length==0) return;
    return filtered[0].committed_date as string;
}
async function getCommitDateTime(project: string) {
    for(const user of generateSettings.gitUsers) {
        const dateTime = await getCommitDateForProject(project, encodeURIComponent(user));
        if(dateTime) return dateTime;
    }
    throw new Error('No commits found for project '+project);
}
function parseName(name: string) {
    if(name.includes("-")) return `"${name}"`;
    return name;
}
Promise.all(yamlContent.items.filter(item=>item.source && item.source.includes('gitlab.rubend.nl')).map(async item=>{
    const project = encodeURIComponent(parse(item.source).path!!.slice(1));
    const dateTime = await getCommitDateTime(project);
    return [item.id, dateTime]
})).then((items)=>{
    const code=`/* This file is auto-generated with codeGen.ts. DO NOT MANUALLY EDIT! */
export default {${items.map(([id, date])=>`
    ${parseName(id)}: new Date("${date}")`).join(',')}
} as Record<string, Date>;`
    writeFileSync('generated/lastUpdated.ts', code, 'utf8')
})
