import path from "node:path";
import { fileURLToPath } from "node:url";
import { includeIgnoreFile } from "@eslint/compat";
import pluginVue from "eslint-plugin-vue";
import vueTsEslintConfig from "@vue/eslint-config-typescript";
import prettierConfig from "@vue/eslint-config-prettier";

const __filename = fileURLToPath(import.meta.url);
const __dirname = path.dirname(__filename);

const gitignorePath = path.resolve(__dirname, ".gitignore");
const ERROR = 2;
export default [
  includeIgnoreFile(gitignorePath),
  {
    ignores: ["**/validate.mts", "**/codeGen.mts", "generated/*"],
  },
  ...pluginVue.configs["flat/recommended"],
  ...vueTsEslintConfig(),
  prettierConfig,
  {
    rules: {
      "vue/multi-word-component-names": "off",
      "@typescript-eslint/consistent-indexed-object-style": ERROR,
      "@typescript-eslint/no-unused-vars": [
        "error",
        {
          args: "all",
          argsIgnorePattern: "^_",
          caughtErrors: "all",
          caughtErrorsIgnorePattern: "^_",
          destructuredArrayIgnorePattern: "^_",
          varsIgnorePattern: "^_",
          ignoreRestSiblings: true,
        },
      ],
      "vue/component-name-in-template-casing": [
        "error",
        "PascalCase",
        {
          registeredComponentsOnly: true,
          ignores: [],
        },
      ],
      "vue/prop-name-casing": ["error", "camelCase"],
      "vue/attribute-hyphenation": ["error", "never"],
      "vue/v-on-event-hyphenation": [
        "error",
        "never",
        {
          autofix: true,
        },
      ],
      "vue/require-prop-types": "off", //TODO remove this
      "vue/no-v-html": "off", //we don't have user input, so this is aafe.
      "@typescript-eslint/no-namespace": "off", //TODO remove this
    },
  },
];
