import {createGenerator, Config} from "ts-json-schema-generator";
import Ajv from "ajv";
// @ts-ignore
import addFormats from "ajv-formats"
// @ts-ignore
import yaml from 'js-yaml';
import {RawFileData} from "./src/types";
import {readFileSync} from "fs";
const ajv = new Ajv({
  strict: true,
  strictRequired: true,
  verbose: true,
  allowUnionTypes: true, // DateType fix;
})
addFormats(ajv)
function validateFile(type: string, filename: string) {
  const config: Config = {
    path: "src/types.ts",
    tsconfig: "tsconfig.json",
    type: type,
  };

  const schema = createGenerator(config).createSchema(config.type);
  const validate = ajv.compile(schema);
  const yamlContent = yaml.load(readFileSync(filename, 'utf8'));
  if(validate(yamlContent)) console.log(`${filename} schema is ok!`)
  else {
    console.log(`${filename} schema is not ok!`)
    console.log(validate.errors)
    process.exit(1)
  }
  return yamlContent;
}
validateFile('GenerateSettings', 'codeGenSettings.yml')
const yamlContent = validateFile('RawFileData', 'data.yml') as RawFileData;
const duplicates=yamlContent.items.map(item=>item.id).filter((e, i, a) => a.indexOf(e) !== i)
if(duplicates.length>0) {
  console.log('duplicate ids in data.yml:', duplicates)
  process.exit(1)
}
const tagIds = yamlContent.tags.map(tag=>tag.id)
const missingTags = yamlContent.items.map(item=>item.tags).flat().map(tag=>tag||"").filter(tag=>tag).filter(tag=>!tagIds.includes(tag))
if(missingTags.length>0) {
  console.log('missing tags in data.yml:', missingTags)
  process.exit(1)
}
